/* HTML Filelist Builder. Version 1.2/REXX (09 Feb 1999)
 * by Anton Dushko (2:5030/611.9)
 * This stuff is FREE. Comments & suggestions are welcome.
 */
                                         
/* (c) message */
Say "HTML Filelist Builder. Version 1.2/REXX (09 Feb 1999)" || "0D0A"x ||,
    "by Anton Dushko (2:5030/611.9)" || "0D0A"x
 
/* Some startup operations */
Trace Off
Address CMD
Numeric Digits 12
Signal On Syntax Name IntError
Signal On Halt Name CtrlBreak
Signal Off Error
Signal Off NoValue
/* RexxUtil used only for file deleting, so porting should be easy */
Call RxFuncAdd "SysLoadFuncs", "RexxUtil", "SysLoadFuncs"
Call SysLoadFuncs

/* Get command line args */
Parse Arg Params

Call ShowMessage "Loading..."

/* Load configuration files */
Call HFBInit
Call AreafileLoad INIVars.AreaConfig
Call TemplateLoad INIVars.HTMLTemplate
Call P2ULoad INIVars.P2UConfig
Call E2ILoad INIVars.E2IConfig

/* Fill constants */
Info.SubAreaLinkTop = INIVars.SubAreaAncTop
Info.SubAreaLinkTopALT = INIVars.SubAreaAncTopALT
Info.SubAreaLinkBottom = INIVars.SubAreaAncBottom
Info.SubAreaLinkBottomALT = INIVars.SubAreaAncBottomALT
Info.AreaLinkIndex = INIVars.FilelistURLBase || LegalizeHTMLName(INIVars.FilelistIndexName)
Info.AreaLinkIndexALT = INIVars.FileListIndexALT

Call ShowMessage "Preprocessing areas..."

/* Preprocess areas - generate area's filelist names & urls */
Do I = 1 To Areas.0

  /* Fill template */
  Info.AreaNumber = I
  Info.AreaName = Areas.I.Name 
  Info.AreaComment = Areas.I.1.Comment
  Info.AreaLevel = Areas.I.Level
  Info.AreaLevelText = Value("INIVars.AreaLevel"Areas.I.Level)

  /* Generate names & alternate links */
  Areas.I.FlName = LegalizeHTMLName(TemplateParse(Template.FilelistName))
  Areas.I.FlURL = INIVars.FilelistURLBase || Areas.I.FlName
  Areas.I.FlName = INIVars.FilelistDiskBase || Areas.I.FlName
  Areas.I.FlALT = TemplateParse(Template.FilelistALT)
  Areas.I.Name = HTMLify(Areas.I.Name)
End I

/* Generate names & alternate links for bound areas */
Areas.0.FlName = LegalizeHTMLName(INIVars.FilelistIndexName)
Areas.0.FlURL = INIVars.FilelistURLBase || Areas.0.FlName
Areas.0.FlName = INIVars.FilelistDiskBase || Areas.0.FlName
Areas.0.FlALT = INIVars.FilelistIndexALT
Areas.I.FlName = LegalizeHTMLName(INIVars.FilelistIndexName)
Areas.I.FlURL = INIVars.FilelistURLBase || Areas.I.FlName
Areas.I.FlName = INIVars.FilelistDiskBase || Areas.I.FlName
Areas.I.FlALT = INIVars.FilelistIndexALT

/* Initialize index statistics */
Index.OfflineFiles = 0
Index.NewFiles = 0
Index.TotalFiles = 0
Index.TotalFileSize = 0

/* Process areas - build filelists */
Do I = 1 To Areas.0

  /* Fill area information block (stage 1) */
  Info.AreaNumber = I
  Info.AreaName = Areas.I.Name 
  Info.AreaComment = Areas.I.1.Comment
  Info.AreaLevel = Areas.I.Level
  Info.AreaLevelText = Value("INIVars.AreaLevel."Areas.I.Level)
  Info.AreaLink = Areas.I.FlURL
  Info.AreaLinkPrevious = Value("Areas." || I - 1 || ".FlURL")
  Info.AreaLinkNext = Value("Areas." || I + 1 || ".FlURL")
  Info.AreaLinkPreviousALT = Value("Areas." || I - 1 || ".FlALT")
  Info.AreaLinkNextALT = Value("Areas." || I + 1 || ".FlALT")

  /* Clear area filelist */
  Drop Files.

  /* Initialize area statistics */
  Areas.I.OfflineFiles = 0
  Areas.I.NewFiles = 0
  Areas.I.TotalFiles = 0
  Areas.I.TotalFileSize = 0

  /* Read files.bbs, file sizes & timestamps, make file urls & icons in
   * all subareas */
  Do J = 1 To Areas.I.Dirs

    /* Skip buggy areas without path */
    If Areas.I.J.Path = "" Then
    Do
      Call PrintMessage "No path to area, skipped:" Areas.I.Name "("Areas.I.J.Comment")"
      Iterate J
    End
   
    Call ShowMessage "["Right(I,3,0)"/"Right(Areas.0,3,0)":"Right(J,2,0)"/"Right(Areas.I.Dirs,2,0)"]" Left(Areas.I.Name "("Areas.I.J.Comment")", 59) "- \"

    /* Add trailing backslash to path */
    Areas.I.J.Path = NormalizePath(Areas.I.J.Path)

    /* Use default filesid name if specific missed */
    If Areas.I.J.FilesID = "" Then
      Call ReadFilesBBS J Areas.I.J.Path || INIVars.FilesID
    Else
      /* Insert path if files.bbs name given without path */
      If FileSpec("Path", Areas.I.J.FilesID) = "" Then
        Call ReadFilesBBS J Areas.I.J.Path || Areas.I.J.FilesID
      Else
        Call ReadFilesBBS J Areas.I.J.FilesID

    /* Make base url for this subarea */
    BaseURL = Path2URL(Areas.I.J.Path)

    Call ShowMessage "["Right(I,3,0)"/"Right(Areas.0,3,0)":"Right(J,2,0)"/"Right(Areas.I.Dirs,2,0)"]" Left(Areas.I.Name "("Areas.I.J.Comment")", 59) "- |"
    
    /* Scan files in this subarea */
    Do K = 1 To Files.J.0
      Areas.I.TotalFiles = Areas.I.TotalFiles + 1

      /* Check if file exists */
      If Stream(Areas.I.J.Path || Files.J.K, "C", "QUERY EXISTS") \= "" Then
      Do
        /* File is online */
        Files.J.K.IsOnline = 1
        Files.J.K.Size = Stream(Areas.I.J.Path || Files.J.K, "C", "QUERY SIZE")
        Areas.I.TotalFileSize = Areas.I.TotalFileSize + Files.J.K.Size
        DateTime = Stream(Areas.I.J.Path || Files.J.K, "C", "QUERY DATETIME")
        Files.J.K.Date = Word(DateTime, 1)
        Files.J.K.Time = Word(DateTime, 2)
        Files.J.K.URL = NormalizeURL(BaseURL || Files.J.K)
        Files.J.K.IsNew = ((Date("B") - FileDate2BaseDate(Files.J.K.Date)) < INIVars.NewDays)
        Areas.I.NewFiles = Areas.I.NewFiles + Files.J.K.IsNew
      End
      Else
      Do
        /* File is offline */
        Files.J.K.IsOnline = 0
        Files.J.K.IsNew = 0
        Areas.I.OfflineFiles = Areas.I.OfflineFiles + 1
        Files.J.K.Size = INIVars.OfflineSize
        Files.J.K.Date = INIVars.OfflineDate
        If INIVars.UseOfflineURL Then
          Files.J.K.URL = INIVars.OfflineURL
        Else
          Files.J.K.URL = NormalizeURL(BaseURL || Files.J.K)
      End

      /* Choose icon for this file */
      Parse Value Ext2Icon(Files.J.K) With Files.J.K.Icon Files.J.K.IconALT
    End K

    /* Set subarea anchor name */
    Areas.I.J.Link = J
  End J

  /* Set anchor names & comments for bound subareas */
  Areas.I.0.Link = INIVars.SubAreaAncTop
  Areas.I.0.Comment = INIVars.SubAreaAncTopALT
  Areas.I.J.Link = INIVars.SubAreaAncBottom
  Areas.I.J.Comment = INIVars.SubAreaAncBottomALT

  /* Fill area information block (stage 2) */
  Info.AreaTotalFiles = NormalizeNumeric(Areas.I.TotalFiles)
  Info.AreaTotalFileSize = NormalizeFileSize(Areas.I.TotalFileSize, INIVars.FileSizeFormatArea)
  Info.AreaNewFiles = NormalizeNumeric(Areas.I.NewFiles)
  Info.AreaOfflineFiles = NormalizeNumeric(Areas.I.OfflineFiles)

  /* Update index statisics */
  Index.OfflineFiles = Index.OfflineFiles + Areas.I.OfflineFiles
  Index.NewFiles = Index.NewFiles + Areas.I.NewFiles
  Index.TotalFiles = Index.TotalFiles + Areas.I.TotalFiles
  Index.TotalFileSize = Index.TotalFileSize + Areas.I.TotalFileSize

  /* Recreate filelist for this area */
  Call SysFileDelete Areas.I.FlName
  If Stream(Areas.I.FlName, "C", "OPEN WRITE") \= "READY:" Then
  Do
    Call PrintMessage "Can't create filelist for" Areas.I.Name
    Iterate I
  End

  /* Generate filelist header */
  Call CharOut Areas.I.FlName, TemplateParse(Template.FilelistHeader)

  /* Generate area header */
  Call CharOut Areas.I.FlName, TemplateParse(Template.AreaHeader)

  /* Generate subareas */
  CurColor = 1
  Do J = 1 To Areas.I.Dirs

    Call ShowMessage "["Right(I,3,0)"/"Right(Areas.0,3,0)":"Right(J,2,0)"/"Right(Areas.I.Dirs,2,0)"]" Left(Areas.I.Name "("Areas.I.J.Comment")", 59) "- /"

    /* Fill subarea information block */
    Info.SubAreaName = Areas.I.J.Comment
    Info.SubAreaLink = Areas.I.J.Link
    Info.SubAreaLinkPrevious = Value("Areas.I." || J - 1 ||".Link")
    Info.SubAreaLinkPreviousALT = Value("Areas.I." || J - 1 || ".Comment")
    Info.SubAreaLinkNext = Value("Areas.I." || J + 1 ||".Link")
    Info.SubAreaLinkNextALT = Value("Areas.I." || J + 1 ||".Comment")

    /* Generate subarea header */
    If J = 1 Then 
      Call CharOut Areas.I.FlName, TemplateParse(Template.FirstSubAreaHeader)
    Else
      Call CharOut Areas.I.FlName, TemplateParse(Template.SubAreaHeader)
  
    /* Reset color sequence if enabled */
    If INIVars.SubAreaColorReset Then
        CurColor = 1

    /* Process files in ths subarea */
    Do K = 1 To Files.J.0
      /* Fill file information block */
      Info.FileIcon = Files.J.K.Icon
      Info.FileIconALT = Files.J.K.IconALT
      Info.FileName = HTMLify(Files.J.K)
      Info.FileURL = Files.J.K.URL
      Info.FileSize = NormalizeFileSize(Files.J.K.Size, INIVars.FileSizeFormatList)
      Info.FileTime = Files.J.K.Time
      Info.FileDate = NormalizeFileDate(Files.J.K.Date)
      Info.FileDesc = Files.J.K.Desc
      Info.FileColor = INIVars.FileColor.CurColor

      /* Choose template */
      Select
        /* Check if this file new */
        When Files.J.K.IsNew Then
        Do
          /* Generate file entry */
          If INIVars.UseNewColor Then
            Info.FileColor = INIVars.NewColor
          Call CharOut Areas.I.FlName, TemplateParse(Template.FileNewEntry)
          /* Change color if enabled */
          If INIVars.NewColorChange Then
          Do
            CurColor = CurColor + 1
            If CurColor > INIVars.FileColors Then
              CurColor = 1
          End
        End
        
        /* Check if this file online */
        When Files.J.K.IsOnline Then
        Do
          /* Generate file entry */
          Call CharOut Areas.I.FlName, TemplateParse(Template.FileEntry)
          /* Change color */
          CurColor = CurColor + 1
          If CurColor > INIVars.FileColors Then
            CurColor = 1
        End
        
        /* Offline file */
        Otherwise
          /* Generate file entry */
          If INIVars.UseOfflineColor Then
            Info.FileColor = INIVars.OfflineColor
          Call CharOut Areas.I.FlName, TemplateParse(Template.OfflineFileEntry)
          /* Change color if enabled */
          If INIVars.OfflineColorChange Then
          Do
            CurColor = CurColor + 1
            If CurColor > INIVars.FileColors Then
              CurColor = 1
          End
      End
    End K

    /* Generate subarea footer */
    Call CharOut Areas.I.FlName, TemplateParse(Template.SubAreaFooter)
  End J

  /* Generate area footer */
  Call CharOut Areas.I.FlName, TemplateParse(Template.AreaFooter)

  /* Generate filelist footer */
  Call CharOut Areas.I.FlName, TemplateParse(Template.FilelistFooter)

  /* Close this area */
  Call Stream Areas.I.FlName, "C", "CLOSE"
End I

Call ShowMessage "Generating index..."

/* Recreate index */
AreaIndexName = INIVars.FilelistDiskBase || INIVars.FileListIndexName
Call SysFileDelete AreaIndexName
If Stream(AreaIndexName, "C", "OPEN WRITE") \= "READY:" Then
Do
  Call PrintMessage "Can't create area index"
  Exit 1
End

/* Generate index page header */
Call CharOut AreaIndexName, TemplateParse(Template.IndexPageHeader)

/* Fill index information */
Info.IndexTotalAreas = Areas.0
Info.IndexNewFiles = NormalizeNumeric(Index.NewFiles)
Info.IndexOfflineFiles = NormalizeNumeric(Index.OfflineFiles)
Info.IndexTotalFiles = NormalizeNumeric(Index.TotalFiles)
Info.IndexTotalFileSize = NormalizeFileSize(Index.TotalFileSize, INIVars.FileSizeFormatIndex)

/* Generate index header */
Call CharOut AreaIndexName, TemplateParse(Template.IndexHeader)

/* Generate index body */
Do I = 1 To Areas.0
  /* Fill area information */
  Info.AreaNumber = I
  Info.AreaName = Areas.I.Name 
  Info.AreaComment = Areas.I.1.Comment
  Info.AreaLevel = Areas.I.Level
  Info.AreaLevelText = Value("INIVars.AreaLevel."Areas.I.Level)
  Info.AreaLevelColor = Value("INIVars.AreaLevelColor."Areas.I.Level)
  Info.AreaLink = Areas.I.FlURL
  Info.AreaTotalFiles = NormalizeNumeric(Areas.I.TotalFiles)
  Info.AreaTotalFileSize = NormalizeFileSize(Areas.I.TotalFileSize, INIVars.FileSizeFormatIndexA)
  Info.AreaNewFiles = NormalizeNumeric(Areas.I.NewFiles)
  Info.AreaOfflineFiles = NormalizeNumeric(Areas.I.OfflineFiles)

  /* Generate index entry */
  Call CharOut AreaIndexName, TemplateParse(Template.IndexEntry)
End I

/* Generate index footer */
Call CharOut AreaIndexName, TemplateParse(Template.IndexFooter)

/* Generate index page header */
Call CharOut AreaIndexName, TemplateParse(Template.IndexPageFooter)

/* Close index */
Call Stream AreaIndexName, "C", "CLOSE"

Call PrintMessage "Done."

/* All OK ! */
Exit 0


/* Supplemental functions */

/* Initialization */

/* Read HFB main initialization file & do some extra initaialization */
HFBInit: Procedure Expose INIVars.

  /* Get location of HFB */
  Parse Source . . SrcName
  HFBPath = FileSpec("D", SrcName) || FileSpec("P", SrcName)
  
  /* Read & process HFB.INI */
  Signal On Syntax Name INIError
  INILine = 0
  INIFile = HFBPath || "HFB.INI"
  Call Stream INIFile, "C", "OPEN READ"
  Do Until Stream(INIFile) <> "READY"
    INILine = INILine + 1
    Parse Value Translate(LineIn(INIFile), " ", "9"x) With Variable Value
    If DataType(Variable, "S") = 1 Then
      Interpret "INIVars." || Strip(Variable) || "=" || Strip(Value)
  End
  Call Stream INIFile, "C", "CLOSE"
  Signal On Syntax Name IntError

  /* Some extra initialization */
  INIVars.FilelistDiskBase = NormalizePath(INIVars.FilelistDiskBase)
  INIVars.LongDesc.Length = Length(INIVars.LongDesc)
Return

/* Initialization file loader's error handler */
INIError:
  Call PrintMessage "INI error in" INIFile ", line" INILine
  Exit -11

/* Load area configuraion file */
AreafileLoad: Procedure Expose Areas. INIVars.
  Parse Arg AreaFile.1
  RD = 1

  /* Clear area array */
  Drop Areas.
  I = 0

  /* Open areafile */
  Call Stream AreaFile.RD, "C", "OPEN READ"

  /* Continue reading from areafiles if we have some of them in stack */
  Do While RD > 0
    /* Read till the end of areafile */
    Do Until Stream(AreaFile.RD) <> "READY"
      Parse Value Translate(Strip(LineIn(AreaFile.RD)), " ", "	") With KeyWord Val
      KeyWord = Translate(KeyWord)
      Val = Strip(Val)
      Select
        /* Kill empty line */
        When KeyWord = "" Then
          Iterate

        /* Begininig of new area */
        When KeyWord = "NAME" Then
        Do
          Areas.I.Dirs = J
          J = 1
          I = I + 1
          /* Initialize new area */
          Areas.I.Name = Val
          Areas.I.Level = INIVars.DefLevel
          Areas.I.1.Path = ""
          Areas.I.1.FilesID = ""
          Areas.I.1.Comment = ""
        End

        /* Additional subareas */
        When KeyWord = "ADD" Then
        Do
          J = J + 1
          Parse Value Val With Areas.I.J.Path","Areas.I.J.FilesID","Areas.I.J.Comment
          Areas.I.J.Comment = HTMLify(Areas.I.J.Comment)
        End

        /* Other keywords */
        When KeyWord = "COMMENT" Then
          Areas.I.1.Comment = HTMLify(Val)
        When KeyWord = "PATH" Then
          Areas.I.1.Path = Val
        When KeyWord = "FILESID" Then
          Areas.I.1.FilesID = Val
        When KeyWord = "LEVEL" Then
          Areas.I.Level = Val

        /* Include another areafile */
        When KeyWord = "INCLUDECFG" Then
        Do
          If FileSpec("N", Val) = Val Then
            Val = FileSpec("D", AreaFile.RD) || FileSpec("P", AreaFile.RD) || Val
          RD = RD + 1
          AreaFile.RD = Strip(Val)
          Call Stream AreaFile.RD, "C", "OPEN READ"
        End

        /* Ignore rest */
        Otherwise
      End
    End

    /* Close this file & return to previous */
    Call Stream AreaFile.RD, "C", "CLOSE"
    RD = RD - 1
  End

  /* Finalize */
  Areas.I.Dirs = J
  Drop Areas.0.
  Areas.0 = I
Return


/* Read Files.BBS or substituting file description file */
ReadFilesBBS: Procedure Expose Files. INIVars.
  Parse Arg J FilesBBS
  
  /* Clear subarea file array */
  Drop Files.J.

  /* Open description file, clear/initialize trash placeholder */
  Call Stream FilesBBS, "C", "OPEN READ"
  I = 0
  Files.J.0.Desc = ""

  /* Read till the end of file */
  Do Until Stream(FilesBBS) <> "READY"
    Line = LineIn(FilesBBS)
    Select
      /* Kill empty line */
      When Strip(Line) = "" Then
        Iterate

      /* Continue long description */
      When Pos(INIVars.LongDesc, Line) = 1 Then
        Files.J.I.Desc = Files.J.I.Desc || INIVars.LongDescCR || WarpNTranslateDesc(SubStr(Line, INIVars.LongDesc.Length + 1))

      /* Begin new file */
      Otherwise
        I = I + 1
        /* '"' on the front fo file name means LFN */
        If Left(Line, 1) = """" Then
        /* LFN */
        Do
          Files.J.I = SubStr(Line, 2, Pos("""", Line, 2) - 1)
          Files.J.I.Desc = WarpNTranslateDesc(SubStr(Line, Pos("""", Line, 2) + 2))
        End
        Else
        /* 8.3 */
        Do
          Files.J.I = Strip(Left(Line, 12))
          Files.J.I.Desc = WarpNTranslateDesc(SubStr(Line, 14))
        End
    End
  End
  Files.J.0 = I

  /* Close file & clear trashcan */
  Call Stream FilesBBS, "C", "CLOSE"
  Drop Files.J.0.Desc
Return

/* Warp description on it's margin */
WarpNTranslateDesc: Procedure Expose INIVars.
  Parse Arg String

  /* Do nothing if warping disabled */
  If INIVars.DescMargin = 0 Then
    Return String

  Result = ""
  Do While Length(String) > INIVars.DescMargin
    /* Try to find whitespace */
    I = LastPos(" ", String, INIVars.DescMargin)
    If I = 0 Then
    Do
      /* Can't find whitespace, warping directly at margin */
      Result = Result || Left(String, INIVars.DescMargin) || INIVars.LongDescCR
      String = SubStr(String, INIVars.DescMargin + 1)
    End
    Else
    Do
      /* Whitespace found, drop it & warp at it's position */
      Result = Result || Left(String, I - 1) || INIVars.LongDescCR 
      String = SubStr(String, I + 1)
    End
  End
Return HTMLify(Result || String)


/* Templates */

/* Load a template set */
TemplateLoad: Procedure Expose Template.
  Parse Arg TPLFile

  /* Open TPL file */
  RC = Stream(TPLFile, "C", "OPEN READ")
  If RC <> "READY:" Then
    Signal TPLError
    
  /* Load & close TPL file */
  AllTemplates = CharIn(TPLFile,, Stream(TPLFile, "C", "QUERY SIZE"))
  Call Stream TPLFile, "C", "CLOSE"
  
  /* Process it */
  Signal On Syntax Name TPLError
  M = 0
  Do Forever
    /* Find begining of template in file */
    N = Pos("0D0A"x || "%+", AllTemplates, M + 1)
    /* Exit if no templates left */
    If N = 0 Then
      Leave
    /* Find end of template */
    M = Pos("%-%" || "0D0A"x, AllTemplates, N + 1)
    If M = 0 Then
      M = Length(AllTemplates)
    /* Find end of template name */
    K = Pos("%" || "0D0A"x, AllTemplates, N + 1)

    /* Gather template body & name */
    Interpret("Template." || SubStr(AllTemplates, N + 4, K - N - 4) || "= SubStr(AllTemplates, K + 3, M - K - 3)") 
  End
  Signal On Syntax Name IntError

Return 

/* Parse a template */
TemplateParse: Procedure Expose Info. INIVars.
  Parse Arg Template, ExtraProcessing
  
  Result = ""
  Do I = 1
    /* Find position of opening escape character */
    J = Pos("%", Template, I)
    /* Set it's position after the end of template if not found */
    If J = 0 Then
      J = Length(Template) + 1

    /* Add text before escape char to result */
    Result = Result || SubStr(Template, I, J - I)

    /* Move begining-of-text pointer */
    I = J + 1
    /* Leave if it falls out of string */
    If I > Length(Template) Then
      Leave I

    /* Find position of closing escape charater */
    J = Pos("%", Template, I)
    /* Leave if it falls out of string */
    If J = 0 Then 
      Leave I

    /* Add escape character to result if macro is empty */
    If I = J Then
    Do
      Result = Result || "%"
      Iterate I
    End

    /* Parse content of macro */
    Parse Value SubStr(Template, I, J - I) With Var","Len","Align","Pad .

    /* Default to left align & space padding */
    If Align = "" Then
      Align = "L"
    If Pad = "" Then
      Pad = " "

    /* Add macro value to result */
    Var = Translate(Var)
    If Symbol("Info." || Var) = "VAR" Then
      /* Select aligning */
      Select
        /* No aligning */
        When Len = "" Then
          Result = Result || Info.Var
        /* Left aligned */
        When Align = "L" Then
          Result = Result || Left(Info.Var, Len, Pad)
        /* Right aligned */
        When Align = "R" Then
          Result = Result || Right(Info.Var, Len, Pad)
        /* Centred */
        When Align = "C" Then
          Result = Result || Center(Info.Var, Len, Pad)
      End

    /* Move begining-of-text pointer */
    I = J
  End I
Return Result

/* TemplateDrop - drops all templates */
TemplateDrop: Procedure Expose Template.
  Drop Template.
Return

/* Template loader's error handler */
TPLError:
  Call SayMessage "TPL error in" TPLFile "-" RC
  Exit -12


/* Bringing things to it's normal/standard/legal state */

/* Add trainling slash to path if missed */
NormalizePath: Procedure
  Parse Arg Path
  If (Right(Path, 1) \= "\") & (Right(Path, 1) \= "/") Then
    Return Path || "\"
Return Path

/* Convert file date to user-defined format */
NormalizeFileDate: Procedure Expose Template. INIVars.
  Parse Arg Info.Month"-"Info.Day"-"Info.Year

  If DataType(Info.Month, "N") & (Info.Month > 0) & (Info.Month < 13) Then
    Info.MonthName = Word(INIVars.MonthNames, Info.Month)
Return TemplateParse(Template.FileDate)

/* Convert file date to basedate */
FileDate2BaseDate: Procedure
   Parse Arg Month"-"Day"-"Year
   
   /* No century given */
   If Year < 100 Then
     If SubStr(Date("S"), 3, 2) < Year Then
       Year = Left(Date("S"), 2) || Year - 100
     Else
       Year = Left(Date("S"), 2) || Year
     
   BaseDate = 365*Year + Year%4 - Year%100 + Year%400 - 366 + Day
   Select
     When Month < 3 Then
        BaseDate = BaseDate + (Month-1)*31
     When Month < 8 Then
        BaseDate = BaseDate + (Month-1)*30 + Month//2 - 2 + (Year//4=0)-(Year//100=0)+(Year//400=0)
     Otherwise
        BaseDate = BaseDate + (Month-1)*30 + (Month-1)//2 - 2 + (Year//4=0)-(Year//100=0)+(Year//400=0)
   End
Return BaseDate

/* Convert file size to user-defined format */
NormalizeFileSize: Procedure
  Parse Arg Size, Divisor DecDigits """"Suffix"""" SectionSize """"Separator""""

  /* Skip non-numeric */
  If DataType(Size, "N") \= 1 Then
    Return Size

  /* Divide */
  Size = Format(Size / Divisor,, DecDigits)
  If Pos(".", Size) = 0 Then
    I = Length(Size) - SectionSize
  Else
    I = Pos(".", Size) - SectionSize - 1

  /* Insert separators */
  Do While I > 0
    Size = Left(Size, I) || Separator || SubStr(Size, I + 1)
    I = I - SectionSize
  End
Return Size || Suffix

/* Insert separators in numerical values */
NormalizeNumeric: Procedure Expose INIVars.
  Parse Arg Num

  I = Length(Num) - INIVars.NumericSectionSize
  Do While I > 0
    Num = Left(Num, I) || INIVars.NumericSeparator || SubStr(Num, I + 1)
    I = I - INIVars.NumericSectionSize
  End
Return Num

/* Convert URL to lowercase and backslash to slash */
NormalizeURL: Procedure
  Parse Arg String
Return Translate(String, XRange("a", "z") || "/", XRange("A", "Z") || "\")

/* Make HTML file name legal */
LegalizeHTMLName: Procedure 
  Parse Arg Name
Return Translate(Name, "0123456789_-." || Copies(XRange("a", "z"), 2),,
                       "0123456789_-." || XRange("a", "z") || XRange("A", "Z") || XRange("00"x, "FF"x), "_")

/* Translate code page & replace HTML special chars (<>&") with it's
 * verbal versions */
HTMLify: Procedure Expose INIVars.
  Parse Arg String

  /* Translate code page */
  String = Translate(String, INIVars.CPTranslateOut, INIVars.CPTranslateIn)
  
  /* Translate special characters */
  J = Verify(String, INIVars.SCTranslateIn, "M", 1)
  Do While J \= 0
    K = Pos(SubStr(String, J, 1), INIVars.SCTranslateIn)
    String = Left(String, J - 1) || INIVars.SCTranslateOut.K || SubStr(String, J + 1)
    I = J + Length(INIVars.SCTranslateOut.K)
    J = Verify(String, INIVars.SCTranslateIn, "M", I)
  End
Return String


/* Path -> URL convertion */

/* Preload P2U translation table */
P2ULoad: Procedure Expose P2UList.
  Parse Arg File

  Call Stream File, "C", "OPEN READ"
  I = 0
  Do Until Stream(File) <> "READY"
    Line = Translate(Strip(Translate(LineIn(File), " ", "	")))
    If (Line = "") | (Left(Line, 1) = ";") Then
      Iterate
    I = I + 1
    Parse Value Line With P2UList.I.Path P2UList.I.URL
    P2UList.I.Path = Strip(P2UList.I.Path)
    P2UList.I.URL = Strip(P2UList.I.URL)
  End
  P2UList.0 = I
  Call Stream File, "C", "CLOSE"
Return

/* Translate path to URL */
Path2URL: Procedure Expose P2UList.
  Parse Upper Arg String

  Do I = 1 To P2UList.0
    PathPos = Pos(P2UList.I.Path, String)
    If PathPos = 0 Then Iterate I
    String = Left(String, PathPos - 1) || P2UList.I.URL ||,
             SubStr(String, PathPos + Length(P2UList.I.Path))
  End
Return String


/* Extention -> Icon translation */

/* Preload E2I translation table */
E2ILoad: Procedure Expose E2IList.
  Parse Arg File

  Call Stream File, "C", "OPEN READ"
  Do Until Stream(File) <> "READY"
    Line = Strip(Translate(LineIn(File), " ", "	"))
    If (Line = "") | (Left(Line, 1) = ";") Then
      Iterate
    Parse Value Line With Extention IconURL IconALT
    If DataType(Extention, "S") = 0 Then
      Iterate
    Interpret("E2IList."Extention".URL = Strip(IconURL)")
    Interpret("E2IList."Extention".ALT = Strip(IconALT)")
  End
  Call Stream File, "C", "CLOSE"
Return

/* Substitute icon for extention  */
Ext2Icon: Procedure Expose E2IList.
  Parse Arg FileName

  If LastPos(".", FileName) = 0 Then
    Extention = "???"
  Else
    Extention = SubStr(FileName, LastPos(".", FileName) + 1)

  If Symbol("E2IList."Extention".URL") \= "VAR" Then
    Extention = "???"
  
Return E2IList.Extention.URL E2IList.Extention.ALT


/* Messages */

ShowMessage: Procedure
  Parse Arg Message
  Call CharOut "CON", "D"x || Copies(" ", 79) || "D"x || Message
Return

PrintMessage: Procedure
  Parse Arg Message
  Call LineOut "CON", "D"x || Copies(" ", 79) || "D"x || Message
Return

SayMessage: Procedure
  Parse Arg Message
  Call LineOut "CON", "0D0A"x || Message
Return


/* General signal handlers */

IntError:
  Call SayMessage "Internal error" RC "at line" SIGL
  Exit -5

CtrlBreak:
  Call SayMessage "Ctrl-Break pressed!"
  Exit -1

/* End of HFB.CMD */
